/* prolog server */

:- use_module(library(sockets)).
:- use_module(library(lists)).

start_server(Port, QueueLength) :-
        socket('AF_INET', Socket),
        socket_bind(Socket, 'AF_INET'(localhost, Port)),
        socket_listen(Socket, QueueLength),
        loop(Socket).

loop(Socket) :-
        socket_accept(Socket, Client, Stream),
        on_exception(RE, get_request(Stream, Request),
		     h1(Stream,Client,RE)),
        on_exception(PE, process(Request, Stream, Client),
	h1(Stream, Client, PE)),
        loop(Socket).

get_request(Stream, Request):-
        read(Stream, Request).

process(bye, Stream, _Client) :-
        close(Stream).
process(Request, Stream, Client) :-
        on_exception(E, user:application_handle(Request,Result), 
	h2(Stream, no, E)),
        return(Stream,Result),
        on_exception(RE, get_request(Stream, Request1),
		     h1(Stream,Client,RE)),
        on_exception(PE, process(Request1, Stream, Client),
	h1(Stream, Client, PE)).

return(Stream,Result) :-
        nonvar(Result), !,
        format(Stream, "~w~nend~n", [Result]),
        flush_output(Stream).
return(_, _).

h1(Stream, C, E) :-
        format(Stream, "error~n",[]),
        format(user_output, "Client: ~w~nError: ~w",[C, E]).

h2(Stream, Ack, E) :-
        format(Stream, "~w~n",[Ack]),
        format(user_output, "Application Error: ~w~n",[E]).

% add_node/2 

application_handle(add_node(Key),Output) :- 
	write('Running add_node...'), nl,
	add_node(Key,Output).
add_node(Key,Output) :- 
	assert(pnode(Key)),
	Output ='Ok'.

% add_before/2

application_handle(add_before(Point1,Point2),Output) :- 
	write('Running add_before...'), nl,
	add_before(Point1, Point2,Output).
add_before(Point1,Point2,Output) :- 
	assert(before_pt(Point1,Point2)),
	Output ='Ok'.

% add_equal/2 

application_handle(add_equal(Point1,Point2),Output) :- 
	write('Running add_equal...'), nl,
	add_equal(Point1, Point2,Output).
add_before(Point1,Point2,Output) :- 
	assert(equal_pt(Point1,Point2)),
	Output ='Ok'.

% check_process/2 

application_handle(check_process(List),Output) :- 
	write('Running check_process2...'), nl,
	check_process(List,Output).
check_process(List,Output) :- 
	list_adder(List, Result),
	L1 = "The sum is: ",
	name(Result,L2),
	append(L1,L2,L3),
	name(Output,L3).

% check_process/1 
application_handle(check_process,Output) :- 
	write('Running check_process1...'), nl,
	check_process(Output).
check_process(Output) :- 
	%there is a problem if prolog can prove an abnormal relationship
	%ab(A,B),
	Output ='Problem!'.

% misc util

list_adder([],0).
list_adder([X|Rest],Z) :-
	list_adder(Rest,Z1),
	Z is X + Z1.

% Temporal Reasoner

equal_pt(Tp,Tp).
before_pt(begin_point(A),end_point(A)) :- pnode(A).
%before_pt(Tp1,Tp3) :-  \+Tp1=Tp2, \+Tp1=Tp3, \+Tp2=Tp3, 
%	before_pt(Tp1,Tp2), before_pt(Tp2,Tp3).

% translate timepoint theory to allen interval relationships
before_a(A,B) :- pnode(A), pnode(B), \+A=B, 
	before_pt(end_point(A), begin_point(B)).
after_a(B,A) :- before_a(A,B).
meets_a(A,B) :- pnode(A), pnode(B), \+A=B,
	equal_pt(end_point(A), begin_point(B)).
met_by_a(B,A) :- meets_a(A,B).
overlaps_a(A,B) :- pnode(A), pnode(B), \+A=B,
	before_pt(begin_point(A), begin_point(B)),
	before_pt(begin_point(B), end_point(A)).
overlapped_by_a(B,A) :- overlaps_a(A,B).
starts_a(A,B) :- pnode(A), pnode(B), \+A=B,
	equal_pt(begin_point(A), begin_point(B)),
	before_pt(end_point(A),end_point(B)).
started_by_a(B,A) :- starts_a(A,B).
during_a(A,B) :- pnode(A), pnode(B), \+A=B,
	before_pt(begin_point(B), begin_point(A)),
	before_pt(end_point(A), end_point(B)).
contains_a(B,A) :- during_a(A,B).
finishes_a(A,B) :- pnode(A), pnode(B), \+A=B,
	before_pt(begin_point(B), begin_point(A)),
	equal_pt(end_point(A), end_point(B)).
finished_by_a(B,A) :- finishes_a(A,B).
equal_a(A,B) :- pnode(A), pnode(B), \+A=B,
	equal_pt(begin_point(A), begin_point(B)),
	equal_pt(end_point(A), end_point(B)).

% encoding of Allen's transitivity table, p17 Reason. about Plans
/* equal_a(A,A).
equal_a(A,C) :- pnode(A), pnode(B), pnode(C), 
	\+A=B, \+A=C, \+B=C, equal_a(A,B), equal_a(B,C).
before_a(A,C) :- pnode(A), pnode(B), pnode(C), 
	\+A=B, \+A=C, \+B=C, before_a(A,B), before_a(B,C).
before_a(A,C) :- pnode(A), pnode(B), pnode(C), 
	\+A=B, \+A=C, \+B=C, before_a(A,B), contains_a(B,C).
before_a(A,C) :- pnode(A), pnode(B), pnode(C), 
	\+A=B, \+A=C, \+B=C, before_a(A,B), overlaps_a(B,C).
before_a(A,C) :- pnode(A), pnode(B), pnode(C), 
	\+A=B, \+A=C, \+B=C, before_a(A,B), meets_a(B,C).
before_a(A,C) :- pnode(A), pnode(B), pnode(C), 
	\+A=B, \+A=C, \+B=C, before_a(A,B), starts_a(B,C).
before_a(A,C) :- pnode(A), pnode(B), pnode(C), 
	\+A=B, \+A=C, \+B=C, before_a(A,B), started_by_a(B,C).
before_a(A,C) :- pnode(A), pnode(B), pnode(C), 
	\+A=B, \+A=C, \+B=C, before_a(A,B), finished_by_a(B,C). 
*/
% table of inverse relationships
/* b  = before
   bi = after
   d  = during
   di = contains 
   o  = overlaps
   oi = overlapped_by
   m  = meets
   mi = met-by
   s  = starts
   si = started-by
   f  = finishes
   fi = finished-by
   e  = equals
*/

inv(b_a(A,B)) :- 
	after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(bi_a(A,B)) :- 
	before_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(bomds_a(A,B)) :- 
	after_c(A,B); contains_c(A,B);
	overlapped_by_c(A,B);
	met_by_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(bioimidf_a(A,B)) :- 
	before_c(A,B); contains_c(A,B);
	overlaps_c(A,B); meets_c(A,B);
	starts_c(A,B); started_by_c(A,B);
	finished_by_c(A,B); equal_c(A,B).
inv(boimidf_a(A,B)) :- 
	after_c(A,B); contains_c(A,B);
	overlaps_c(A,B); meets_c(A,B);
	starts_c(A,B); started_by_c(A,B);
	finished_by_c(A,B); equal_c(A,B). 
inv(d_a(A,B)) :- 
	before_c(A,B); after_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(bomdifi_a(A,B)) :- 
	after_c(A,B); during_c(A,B);
	overlapped_by_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); equal_c(A,B).
inv(bioidimisi_a(A,B)) :- 
	before_c(A,B); during_c(A,B);
	overlaps_c(A,B); meets_c(A,B);
	starts_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(ooiddie_a(A,B)) :- 
	before_c(A,B); after_c(A,B);
	meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B).
inv(di_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(oidifi_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during(A,B);
	overlaps_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); equal_c(A,B).
inv(oidisi_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B);
	overlaps_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(odifi_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B);
	overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); equal_c(A,B).
inv(ods_a(A,B)) :- 
	before_c(A,B); after_c(A,B); contains_c(A,B);
	overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(bom_a(A,B)) :- 
	after_c(A,B); during_c(A,B); contains_c(A,B);
	overlapped_by_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(o_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(dso_a(A,B)) :- 
	before_c(A,B); after_c(A,B); contains_c(A,B);
	overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(oidf_a(A,B)) :- 
	before_c(A,B); after_c(A,B); contains_c(A,B);
	overlaps_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finished_by_c(A,B); equal_c(A,B).
inv(bioimi_a(A,B)) :- 
	after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); meets_c(A,B);
	starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(oi_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(ffie_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B).
inv(m_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(bioi_a(A,B)) :- 
	before_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(ssie_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); 
	finishes_c(A,B); finished_by_c(A,B).
inv(mi_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(mib_a(A,B)) :- 
	after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(df_a(A,B)) :- 
	before_c(A,B); after_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finished_by_c(A,B); equal_c(A,B).
inv(si_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B);
	finishes_c(A,B); finished_by_c(A,B); equal_c(A,B).
inv(f_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finished_by_c(A,B); equal_c(A,B).
inv(e_a(A,B)) :- 
	before_c(A,B); after_c(A,B); during_c(A,B); contains_c(A,B);
	overlaps_c(A,B); overlapped_by_c(A,B); meets_c(A,B);
	met_by_c(A,B); starts_c(A,B); started_by_c(A,B);
	finishes_c(A,B); finished_by_c(A,B).

%blank template of all 13 relations.
%inv(_a(A,B)) :-
%	before_a(A,B); after_a(A,B); during_a(A,B); contains_a(A,B);
%	overlaps_a(A,B); overlapped_by_a(A,B); meets_a(A,B);
%	met_by_a(A,B); starts_a(A,B); started_by_a(A,B);
%	finishes_a(A,B); finished_by_a(A,B); equal_a(A,B).

% abnormal relationships 
ab(A,C) :- pnode(A), pnode(B), \+A=B, pnode(C), \+A=C, \+B=C, ab(A,B,C).

%row1
ab(A,B,C) :- before_a(A,B), before_a(B,C), inv(b_a(A,C)), 
	     explain(b_a,A,'before',B,'before',C).
ab(A,B,C) :- before_a(A,B), during_a(B,C), inv(bomds_a(A,C)),
	     explain(bomds_a,A,'before',B,'during',C).
ab(A,B,C) :- before_a(A,B), contains_a(B,C), inv(b_a(A,C)), 
	     explain(b_a,A,'before',B,'contains',C).
ab(A,B,C) :- before_a(A,B), overlaps_a(B,C), inv(b_a(A,C)), 
	     explain(b_a,A,'before',B,'overlaps',C).
ab(A,B,C) :- before_a(A,B), overlapped_by_a(B,C), inv(bomds_a(A,C)),
	     explain(bomds_a,A,'before',B,'overlapped by',C).
ab(A,B,C) :- before_a(A,B), meets_a(B,C), inv(b_a(A,C)),
	     explain(b_a,A,'before',B,'meets',C).
ab(A,B,C) :- before_a(A,B), met_by_a(B,C), inv(bomds_a(A,C)),
	     explain(bomds_a,A,'before',B,'met by',C).
ab(A,B,C) :- before_a(A,B), starts_a(B,C), inv(b_a(A,C)),
	     explain(b_a,A,'before',B,'starts',C).
ab(A,B,C) :- before_a(A,B), started_by_a(B,C), inv(b_a(A,C)),
	     explain(b_a,A,'before',B,'started by',C).
ab(A,B,C) :- before_a(A,B), finishes_a(B,C), inv(bomds_a(A,C)),
	     explain(bomds_a,A,'before',B,'finishes',C).
ab(A,B,C) :- before_a(A,B), finished_by_a(B,C), inv(b_a(A,C)),
	     explain(b_a,A,'before',B,'finished by',C).

%row2
ab(A,B,C) :- after_a(A,B), after_a(B,C), inv(bi_a(A,C)),
	     explain(bi_a,A,'after',B,'after',C).
ab(A,B,C) :- after_a(A,B), during_a(B,C), inv(bioimidf_a(A,C)),
	     explain(bioimidf_a,A,'after',B,'during',C).
ab(A,B,C) :- after_a(A,B), contains_a(B,C), inv(bi_a(A,C)),
	     explain(bi_a,A,'after',B,'contains',C).
ab(A,B,C) :- after_a(A,B), overlaps_a(B,C), inv(bioimidf_a(A,C)),
	     explain(bioimidf_a,A,'after',B,'overlaps',C).
ab(A,B,C) :- after_a(A,B), overlapped_by_a(B,C), inv(bi_a(A,C)),
	     explain(bi_a,A,'after',B,'overlapped by',C).
ab(A,B,C) :- after_a(A,B), meets_a(B,C), inv(bioimidf_a(A,C)),
	     explain(bioimidf_a,A,'after',B,'meets',C).
ab(A,B,C) :- after_a(A,B), met_by_a(B,C), inv(bi_a(A,C)),
	     explain(bi_a,A,'after',B,'met by',C).
ab(A,B,C) :- after_a(A,B), starts_a(B,C), inv(boimidf_a(A,C)),
	     explain(boimidf_a,A,'after',B,'starts',C).
ab(A,B,C) :- after_a(A,B), started_by_a(B,C), inv(bi_a(A,C)),
	     explain(bi_a,A,'after',B,'started by',C).
ab(A,B,C) :- after_a(A,B), finishes_a(B,C), inv(bi_a(A,C)),
	     explain(bi_a,A,'after',B,'finishes',C).
ab(A,B,C) :- after_a(A,B), finished_by_a(B,C), inv(bi_a(A,C)),
	     explain(bi_a,A,'after',B,'finished by',C).
%row3
ab(A,B,C) :- during_a(A,B), before_a(B,C), inv(b_a(A,C)), 
	     explain(b_a,A,'during',B,'before',C).
ab(A,B,C) :- during_a(A,B), after_a(B,C), inv(bi_a(A,C)),
             explain(bi_a,A,'during',B,'after',C).
ab(A,B,C) :- during_a(A,B), during_a(B,C), inv(d_a(A,C)),
             explain(d_a,A,'during',B,'during',C).
ab(A,B,C) :- during_a(A,B), overlaps_a(B,C), inv(bomds_a(A,C)),
             explain(bomds_a,A,'during',B,'overlaps',C).
ab(A,B,C) :- during_a(A,B), overlapped_by_a(B,C), inv(bioimidf_a(A,C)),
             explain(bioimidf_a,A,'during',B,'overlapped by',C).
ab(A,B,C) :- during_a(A,B), meets_a(B,C), inv(b_a(A,C)),
             explain(b_a,A,'during',B,'meets',C).
ab(A,B,C) :- during_a(A,B), met_by_a(B,C), inv(bi_a(A,C)),
             explain(bi_a,A,'during',B,'met by',C).
ab(A,B,C) :- during_a(A,B), starts_a(B,C), inv(d_a(A,C)),
             explain(d_a,A,'during',B,'starts',C).
ab(A,B,C) :- during_a(A,B), started_by_a(B,C), inv(bioimidf_a(A,C)),
             explain(bioimidf_a,A,'during',B,'started by',C).
ab(A,B,C) :- during_a(A,B), finishes_a(B,C), inv(d_a(A,C)),
             explain(d_a,A,'during',B,'finishes',C).
ab(A,B,C) :- during_a(A,B), finished_by_a(B,C), inv(bomds_a(A,C)),
             explain(bomds_a,A,'during',B,'finished by',C).

%row4
ab(A,B,C) :- contains_a(A,B), before_a(B,C), inv(bomdifi_a(A,C)),
             explain(bomdifi_a,A,'contains',B,'before',C).
ab(A,B,C) :- contains_a(A,B), after_a(B,C), inv(bioidimisi_a(A,C)),
             explain(bioidimisi_a,A,'contains',B,'after',C).
ab(A,B,C) :- contains_a(A,B), contains_a(B,C), inv(ooiddie_a(A,C)),
             explain(ooiddie_a,A,'contains',B,'contains',C).
ab(A,B,C) :- contains_a(A,B), during_a(B,C), inv(di_a(A,C)),
             explain(di_a,A,'contains',B,'during',C).
ab(A,B,C) :- contains_a(A,B), overlaps_a(B,C), inv(oidifi_a(A,C)),
             explain(oidifi_a,A,'contains',B,'overlaps',C).
ab(A,B,C) :- contains_a(A,B), overlapped_by_a(B,C), inv(oidisi_a(A,C)),
             explain(oidisi_a,A,'contains',B,'overlapped by',C).
ab(A,B,C) :- contains_a(A,B), meets_a(B,C), inv(odifi_a(A,C)),
             explain(oidifi_a,A,'contains',B,'meets',C).
ab(A,B,C) :- contains_a(A,B), met_by_a(B,C), inv(oidisi_a(A,C)),
             explain(oidisi_a,A,'contains',B,'met by',C).
ab(A,B,C) :- contains_a(A,B), starts_a(B,C), inv(odifi_a(A,C)),
             explain(odifi_a,A,'contains',B,'starts',C).
ab(A,B,C) :- contains_a(A,B), started_by_a(B,C), inv(di_a(A,C)),
             explain(di_a,A,'contains',B,'started by',C).
ab(A,B,C) :- contains_a(A,B), finishes_a(B,C), inv(oidisi_a(A,C)),
             explain(oidisi_a,A,'contains',B,'finishes',C).
ab(A,B,C) :- contains_a(A,B), finished_by_a(B,C), inv(di_a(A,C)),
             explain(di_a,A,'contains',B,'finished by',C).

%row5
ab(A,B,C) :- overlaps_a(A,B), before_a(B,C), inv(b_a(A,C)),
             explain(b_a,A,'overlaps',B,'before',C).
ab(A,B,C) :- overlaps_a(A,B), after_a(B,C), inv(bioidimisi_a(A,C)),
             explain(bioidimisi_a,A,'overlaps',B,'after',C).
ab(A,B,C) :- overlaps_a(A,B), contains_a(B,C), inv(ods_a(A,C)),
             explain(ods_a,A,'overlaps',B,'contains',C).
ab(A,B,C) :- overlaps_a(A,B), during_a(B,C), inv(bomdifi_a(A,C)),
             explain(bomdifi_a,A,'overlaps',B,'during',C).
ab(A,B,C) :- overlaps_a(A,B), overlaps_a(B,C), inv(bom_a(A,C)),
             explain(bom_a,A,'overlaps',B,'overlaps',C).
ab(A,B,C) :- overlaps_a(A,B), overlapped_by_a(B,C), inv(ooiddie_a(A,C)),
             explain(ooiddie_a,A,'overlaps',B,'overlapped by',C).
ab(A,B,C) :- overlaps_a(A,B), meets_a(B,C), inv(b_a(A,C)),
             explain(b_a,A,'overlaps',B,'meets',C).
ab(A,B,C) :- overlaps_a(A,B), met_by_a(B,C), inv(oidisi_a(A,C)),
             explain(oidisi_a,A,'overlaps',B,'met by',C).
ab(A,B,C) :- overlaps_a(A,B), starts_a(B,C), inv(o_a(A,C)),
             explain(o_a,A,'overlaps',B,'starts',C).
ab(A,B,C) :- overlaps_a(A,B), started_by_a(B,C), inv(odifi_a(A,C)),
             explain(odifi_a,A,'overlaps',B,'started by',C).
ab(A,B,C) :- overlaps_a(A,B), finishes_a(B,C), inv(dso_a(A,C)),
             explain(dso_a,A,'overlaps',B,'finishes',C).
ab(A,B,C) :- overlaps_a(A,B), finished_by_a(B,C), inv(bom_a(A,C)),
             explain(bom_a,A,'overlaps',B,'finished by',C).
%row6
ab(A,B,C) :- overlapped_by_a(A,B), before_a(B,C), inv(bomdifi_a(A,C)),
             explain(bomdifi_a,A,'overlapped by',B,'before',C).
ab(A,B,C) :- overlapped_by_a(A,B), after_a(B,C), inv(bi_a(A,C)),
             explain(bi_a,A,'overlapped by',B,'after',C).
ab(A,B,C) :- overlapped_by_a(A,B), contains_a(B,C), inv(oidf_a(A,C)),
             explain(oidf_a,A,'overlapped by',B,'contains',C).
ab(A,B,C) :- overlapped_by_a(A,B), during_a(B,C), inv(bioidimisi_a(A,C)),
             explain(bioidimisi_a,A,'overlapped by',B,'during',C).
ab(A,B,C) :- overlapped_by_a(A,B), overlaps_a(B,C), inv(ooiddie_a(A,C)),
             explain(ooiddie_a,A,'overlapped by',B,'overlaps',C).
ab(A,B,C) :- overlapped_by_a(A,B), overlapped_by_a(B,C), inv(bioimi_a(A,C)),
             explain(bioimi_a,A,'overlapped by',B,'overlapped by',C).
ab(A,B,C) :- overlapped_by_a(A,B), meets_a(B,C), inv(odifi_a(A,C)),
             explain(odifi_a,A,'overlapped by',B,'meets',C).
ab(A,B,C) :- overlapped_by_a(A,B), met_by_a(B,C), inv(bi_a(A,C)),
             explain(bi_a,A,'overlapped by',B,'met by',C).
ab(A,B,C) :- overlapped_by_a(A,B), starts_a(B,C), inv(oidf_a(A,C)),
             explain(oidf_a,A,'overlapped by',B,'starts',C).
ab(A,B,C) :- overlapped_by_a(A,B), started_by_a(B,C), inv(bioimi_a(A,C)),
             explain(bioimi_a,A,'overlapped by',B,'started by',C).
ab(A,B,C) :- overlapped_by_a(A,B), finishes_a(B,C), inv(oi_a(A,C)),
             explain(oi_a,A,'overlapped by',B,'finishes',C).
ab(A,B,C) :- overlapped_by_a(A,B), finished_by_a(B,C), inv(oidisi_a(A,C)),
             explain(oidisi_a,A,'overlapped by',B,'finished by',C).

%row7
ab(A,B,C) :- meets_a(A,B), before_a(B,C), inv(b_a(A,C)),
             explain(b_a,A,'meets',B,'before',C).
ab(A,B,C) :- meets_a(A,B), after_a(B,C), inv(bioidimisi_a(A,C)),
             explain(bioidimisi_a,A,'meets',B,'after',C).
ab(A,B,C) :- meets_a(A,B), contains_a(B,C), inv(ods_a(A,C)),
             explain(ods_a,A,'meets',B,'contains',C).
ab(A,B,C) :- meets_a(A,B), during_a(B,C), inv(b_a(A,C)),
             explain(b_a,A,'meets',B,'during',C).
ab(A,B,C) :- meets_a(A,B), overlaps_a(B,C), inv(b_a(A,C)),
             explain(b_a,A,'meets',B,'overlaps',C).
ab(A,B,C) :- meets_a(A,B), overlapped_by_a(B,C), inv(ods_a(A,C)),
             explain(ods_a,A,'meets',B,'overlapped by',C).
ab(A,B,C) :- meets_a(A,B), meets_a(B,C), inv(b_a(A,C)),
             explain(b_a,A,'meets',B,'meets',C).
ab(A,B,C) :- meets_a(A,B), met_by_a(B,C), inv(ffie_a(A,C)),
             explain(ffie_a,A,'meets',B,'met by',C).
ab(A,B,C) :- meets_a(A,B), starts_a(B,C), inv(m_a(A,C)),
             explain(m_a,A,'meets',B,'starts',C).
ab(A,B,C) :- meets_a(A,B), started_by_a(B,C), inv(m_a(A,C)),
             explain(m_a,A,'meets',B,'started by',C).
ab(A,B,C) :- meets_a(A,B), finishes_a(B,C), inv(dso_a(A,C)),
             explain(dso_a,A,'meets',B,'finishes',C).
ab(A,B,C) :- meets_a(A,B), finished_by_a(B,C), inv(b_a(A,C)),
             explain(b_a,A,'meets',B,'finished by',C).
%row8
ab(A,B,C) :- met_by_a(A,B), before_a(B,C), inv(bomdifi_a(A,C)),
             explain(bomdifi_a,A,'met by',B,'before',C).
ab(A,B,C) :- met_by_a(A,B), after_a(B,C), inv(bi_a(A,C)),
             explain(bi_a,A,'met by',B,'after',C).
ab(A,B,C) :- met_by_a(A,B), contains_a(B,C), inv(oidf_a(A,C)),
             explain(oidf_a,A,'met by',B,'contains',C).
ab(A,B,C) :- met_by_a(A,B), during_a(B,C), inv(bi_a(A,C)),
             explain(bi_a,A,'met by',B,'during',C).
ab(A,B,C) :- met_by_a(A,B), overlaps_a(B,C), inv(oidf_a(A,C)),
             explain(oidf_a,A,'met by',B,'overlaps',C).
ab(A,B,C) :- met_by_a(A,B), overlapped_by_a(B,C), inv(bioi_a(A,C)),
             explain(bioi_a,A,'met by',B,'overlapped by',C).
ab(A,B,C) :- met_by_a(A,B), meets_a(B,C), inv(ssie(A,C)),
             explain(ssie_a,A,'met by',B,'meets',C).
ab(A,B,C) :- met_by_a(A,B), met_by_a(B,C), inv(bi_a(A,C)),
             explain(bi_a,A,'met by',B,'met by',C).
ab(A,B,C) :- met_by_a(A,B), starts_a(B,C), inv(oidf_a(A,C)),
             explain(oidf_a,A,'met by',B,'starts',C).
ab(A,B,C) :- met_by_a(A,B), started_by_a(B,C), inv(bi_a(A,C)),
             explain(bi_a,A,'met by',B,'started by',C).
ab(A,B,C) :- met_by_a(A,B), finishes_a(B,C), inv(mi_a(A,C)),
             explain(mi_a,A,'met by',B,'finishes',C).
ab(A,B,C) :- met_by_a(A,B), finished_by_a(B,C), inv(mib_a(A,C)),
             explain(mib_a,A,'met by',B,'finished by',C).

ab(A,B,C) :- equal_a(A,B), equal_a(B,C), inv(e_a(A,C)),
             explain(e_a,A,'equal',B,'equal',C).

%%%%%%%%%%%%%%%%%%%%% Error Reporting Structures _c "check" predicates

after_c(A,B) :- after_a(A,B), write('Error! '), write(A), 
        write(' has been defined as being after '), write(B), write('.'), nl.
before_c(A,B) :- before_a(A,B), write('Error! '), write(A), 
        write(' has been defined as being before '), write(B), write('.'), nl.
meets_c(A,B) :- meets_a(A,B), write('Error! '), write(A), 
        write(' has been defined as meeting '), write(B), write('.'), nl.
met_by_c(A,B) :- met_by_a(A,B), write('Error! '), write(A), 
        write(' has been defined as being met by '), write(B), write('.'), nl.
overlaps_c(A,B) :- overlaps_a(A,B), write('Error! '), write(A), 
        write(' has been defined as overlapping '), write(B), write('.'), nl.
overlapped_by_c(A,B) :- overlapped_by_a(A,B), write('Error! '), write(A), 
        write(' has been defined as being overlapped by '), 
	write(B), write('.'), nl.
starts_c(A,B) :- starts_a(A,B), write('Error! '), write(A), 
        write(' has been defined as starting '), write(B), write('.'), nl.
started_by_c(A,B) :- started_by_a(A,B), write('Error! '), write(A), 
        write(' has been defined as being started by '),
	write(B), write('.'), nl.
during_c(A,B) :- during_a(A,B), write('Error! '), write(A), 
        write(' has been defined as being during '), write(B), write('.'), nl.
contains_c(A,B) :- contains_a(A,B), write('Error! '), write(A), 
        write(' has been defined as containing '), write(B), write('.'), nl.
finishes_c(A,B) :- finishes_a(A,B), write('Error! '), write(A), 
        write(' has been defined as finishing '), write(B), write('.'), nl.
finished_by_c(A,B) :- finished_by_a(A,B), write('Error! '), write(A), 
        write(' has been defined as being finished by '),
	write(B), write('.'), nl.
equal_c(A,B) :- equal_a(A,B), write('Error! '), write(A), 
        write(' has been defined as being equal to '),
	write(B), write('.'), nl.

%%%%%%%%%%%%%%%%%%%%% Error Explanation Structures - explain predicates

explain(b_a,A,ArelB,B,BrelC,C) :-
        standard_explain(A,ArelB,B,BrelC,C,' can only be before ').
explain(bi_a,A,ArelB,B,BrelC,C) :- 
        standard_explain(A,ArelB,B,BrelC,C,' can only be after ').
explain(bomds_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be before, overlapping, meeting, starting, or during '). 
explain(bioimidf_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be after, overlapped by, met by, during, or finishing '). 
explain(boimidf_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be before, overlapped by, met by, during, or finishing '). 
explain(d_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,' can only be during '). 
explain(bomdifi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be before, overlapping, meeting, containing, or finished by '). 
explain(bioidimisi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be after, overlapped by, containing, met by, or started by ').
explain(ooiddie_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be overlapping, overlapped by, during, containing, or equal to ').
explain(di_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be containing ').
explain(oidifi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be overlapped by, containing, or finished by ').
explain(oidisi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be overlapped by, containing, or started by ').
explain(odifi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be overlapping, containing, or finished by ').
explain(ods_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be ovelapping, during, or starting ').
explain(bom_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be before, overlapping, or meeting ').
explain(o_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be overlapping ').
explain(dso_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be during, starting, or overlapping ').
explain(oidf_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be overlapped by, during, or finishing ').
explain(bioimi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be after, overlapped by, or met by ').
explain(oi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be overlapped by ').
explain(ffie_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be finishing, finished by, or equal to ').
explain(m_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be meeting ').
explain(bioi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be after or overlapped by ').
explain(ssie_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be starting, started by, or equal to ').
explain(mi_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be met by ').
explain(mib_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be met by or before ').
explain(df_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be during or finishing ').
explain(si_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be started by ').
explain(f_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be finishing ').
explain(e_a,A,ArelB,B,BrelC,C) :- 
	standard_explain(A,ArelB,B,BrelC,C,
	' can only be equal to ').
standard_explain(A,ArelB,B,BrelC,C,Alts) :-
	write('If '), write(A), write(' is '), write(ArelB), write(' '),
	write(B), write(' and '), write(B), write(' is '), write(BrelC),
	write(' '), write(C), write(' then '), write(A), 
	write(Alts), write(C), write('.'), nl.

%%%%%%%%%%%%%%%%%%%%% Problem Instance Data

%pnode(acta).
%pnode(actb).
%pnode(actc).
%pnode(actd).
%pnode(acte).
%pnode(actf).

% a is before b
%before_pt(end_point(acta),begin_point(actb)).
% b is during c
%before_pt(begin_point(actc),begin_point(actb)).
%before_pt(end_point(actb),end_point(actc)).
%broken: a is after c
%before_pt(end_point(actc),begin_point(acta)).
%ok: c is after a
%before_pt(end_point(acta),begin_point(actc)).

% legal trans. d is before e, d is before a.
%before_pt(end_point(actd),begin_point(acte)).
%before_pt(end_point(actd),begin_point(acta)).
%before_pt(end_point(acte),begin_point(actf)).
% oops, error! d cannot be after f
%before_pt(end_point(actf),begin_point(actd)).

% go!
%:- start_server(6789,1024).